@@ -1,4 +1,4 @@
-gfun <- function(x,y,theta) {
+gfun <- function(xm,y,theta) {
 #
 # Function gfun.  To calculate the probabilites gamma[i,j]
 # that observation i corresponds to state j for the Gaussian model.
@@ -13,12 +13,13 @@ nr     <- nrow(theta)
 sigsq  <- theta[nr-1,]
 lambda <- theta[nr,]
 beta   <- theta[-c(nr-1,nr),]
-n      <- nrow(x)
-
-yhat   <- x%*%beta
-fff    <- matrix(dnorm(t(y-yhat)/sqrt(sigsq)),K,n)/sqrt(sigsq)
-
-hhh    <- t(lambda*fff)
+n      <- nrow(xm)
+yhat   <- xm%*%beta
+fff    <- t(matrix(dnorm(t(y-yhat)/sqrt(sigsq),log=TRUE),K,n)
+            -log(sqrt(sigsq)))
+mlp    <- apply(fff,1,max)
+ggg    <- exp(fff - mlp)
+hhh    <- t(lambda*t(ggg))
 ll     <- apply(hhh,1,sum)
-list(gamma=hhh/ll,log.like=sum(log(ll)))
+list(gamma=hhh/ll,log.like=sum(log(ll)+mlp))
 }
